@@ -1,6 +1,6 @@
 /* MAPIStoreTable.m - this file is part of SOGo
  *
- * Copyright (C) 2010 Inverse inc
+ * Copyright (C) 2010-2012 Inverse inc
  *
  * Author: Wolfgang Sourdeau <wsourdeau@inverse.ca>
  *
@@ -274,7 +274,7 @@ static Class NSDataK, NSStringK;
 {
   if ((self = [super init]))
     {
-      [self logWithFormat: @"-init"];
+      // [self logWithFormat: @"-init"];
       tableType = MAPISTORE_MESSAGE_TABLE;
       handleId = -1;
 
@@ -307,21 +307,21 @@ static Class NSDataK, NSStringK;
   return self;
 }
 
-- (id) retain
-{
-  [self logWithFormat: @"-retain"];
-  return [super retain];
-}
+// - (id) retain
+// {
+//   [self logWithFormat: @"-retain"];
+//   return [super retain];
+// }
 
-- (void) release
-{
-  [self logWithFormat: @"-release"];
-  [super release];
-}
+// - (void) release
+// {
+//   [self logWithFormat: @"-release"];
+//   [super release];
+// }
 
 - (void) dealloc
 {
-  [self logWithFormat: @"-dealloc"];
+  // [self logWithFormat: @"-dealloc"];
   if (columns)
     NSZoneFree (NULL, columns);
   [currentChild release];
@@ -375,12 +375,12 @@ static Class NSDataK, NSStringK;
 
 - (void) setRestrictions: (const struct mapi_SRestriction *) res
 {
-  EOQualifier *oldRestriction;
+  //EOQualifier *oldRestriction;
 
   // [self logWithFormat: @"set restriction to (table type: %d): %@",
   // 	type, MAPIStringForRestriction (res)];
 
-  oldRestriction = restriction;
+  //oldRestriction = restriction;
   [restriction autorelease];
   if (res)
     restrictionState = [self evaluateRestriction: res
@@ -396,11 +396,11 @@ static Class NSDataK, NSStringK;
   // FIXME: we should not flush the caches if the restrictions matches
   [self cleanupCaches];
   
-  if (restriction)
-    [self logWithFormat: @"restriction set to EOQualifier: %@",
-	  restriction];
-  else if (oldRestriction)
-    [self logWithFormat: @"restriction unset (was %@)", oldRestriction];
+  //if (restriction)
+  //  [self logWithFormat: @"restriction set to EOQualifier: %@",
+//	  restriction];
+ // else if (oldRestriction)
+   // [self logWithFormat: @"restriction unset (was %@)", oldRestriction];
 }
 
 - (MAPIRestrictionState) evaluateNotRestriction: (struct mapi_SNotRestriction *) res
@@ -841,8 +841,7 @@ static Class NSDataK, NSStringK;
   struct mapistore_property_data *rowData;
   int rc;
 
-  child = [self childAtRowID: rowId
-                forQueryType: queryType];
+  child = [self childAtRowID: rowId forQueryType: queryType];
   if (child)
     {
       rowData = talloc_array(memCtx, struct mapistore_property_data, columnsCount);
@@ -874,61 +873,6 @@ static Class NSDataK, NSStringK;
   return MAPISTORE_SUCCESS;
 }
 
-- (void) notifyChangesForChild: (MAPIStoreObject *) child
-{
-  NSUInteger currentChildRow, newChildRow;
-  NSArray *list;
-  NSString *childName;
-  struct mapistore_table_notification_parameters notif_parameters;
-  struct mapistore_context *mstoreCtx;
-
-  mstoreCtx = [[container context] connectionInfo]->mstore_ctx;
-
-  notif_parameters.table_type = tableType;
-  notif_parameters.handle = handleId;
-  notif_parameters.folder_id = [container objectId];
-  notif_parameters.object_id = [child objectId];
-  notif_parameters.instance_id = 0; /* TODO: always 0 ? */
-
-  childName = [child nameInContainer];
-  list = [self restrictedChildKeys];
-  currentChildRow = [list indexOfObject: childName];
-  notif_parameters.row_id = currentChildRow;
-
-  [self cleanupCaches];
-  list = [self restrictedChildKeys];
-  newChildRow = [list indexOfObject: childName];
-
-  if (currentChildRow == NSNotFound)
-    {
-      if (newChildRow != NSNotFound)
-        {
-          notif_parameters.row_id = newChildRow;
-          mapistore_push_notification (mstoreCtx,
-                                       MAPISTORE_TABLE,
-                                       MAPISTORE_OBJECT_CREATED,
-                                       &notif_parameters);
-        }
-    }
-  else
-    {
-      if (newChildRow == NSNotFound)
-        mapistore_push_notification (mstoreCtx,
-                                     MAPISTORE_TABLE,
-                                     MAPISTORE_OBJECT_DELETED,
-                                     &notif_parameters);
-      else
-        {
-          /* the fact that the row order has changed has no impact here */
-          notif_parameters.row_id = newChildRow;
-          mapistore_push_notification (mstoreCtx,
-                                       MAPISTORE_TABLE,
-                                       MAPISTORE_OBJECT_MODIFIED,
-                                       &notif_parameters);
-        }
-    }
-}
-
 /* subclasses */
 - (NSString *) backendIdentifierForProperty: (enum MAPITAGS) property
 {
