@@ -27,15 +27,6 @@
 
 #import "GCSSpecialQueries.h"
 
-@interface GCSPostgreSQLSpecialQueries : GCSSpecialQueries
-@end
-
-@interface GCSMySQLSpecialQueries : GCSSpecialQueries
-@end
-
-@interface GCSOracleSpecialQueries : GCSSpecialQueries
-@end
-
 @implementation EOAdaptorChannel (GCSSpecialQueries)
 
 - (GCSSpecialQueries *) specialQueries
@@ -119,6 +110,14 @@
   return nil;
 }
 
+- (NSString *) updateCPathInFolderInfo: (NSString *) tableName
+                            withCPath2: (NSString *) c_path2
+{
+  [self subclassResponsibility: _cmd];
+
+  return nil;
+}
+
 
 @end
 
@@ -163,7 +162,7 @@
 {
   static NSString *sqlFolderFormat
     = (@"CREATE TABLE %@ (\n"
-       @"  c_name VARCHAR (255) NOT NULL PRIMARY KEY,\n"
+       @"  c_name VARCHAR (255) PRIMARY KEY,\n"
        @"  c_content TEXT NOT NULL,\n"
        @"  c_creationdate INT4 NOT NULL,\n"
        @"  c_lastmodified INT4 NOT NULL,\n"
@@ -189,8 +188,8 @@
 - (NSString *) createSessionsFolderWithName: (NSString *) tableName
 {
   static NSString *sqlFolderFormat
-    = (@"CREATE TABLE %@ (" 
-       @" c_id VARCHAR(255) NOT NULL PRIMARY KEY,"
+    = (@"CREATE TABLE %@ ("
+       @" c_id VARCHAR(255) PRIMARY KEY,"
        @" c_value VARCHAR(255) NOT NULL,"
        @" c_creationdate INT4 NOT NULL,"
        @" c_lastseen INT4 NOT NULL)");
@@ -214,6 +213,17 @@
   return types;
 }
 
+- (NSString *) updateCPathInFolderInfo: (NSString *) tableName
+                            withCPath2: (NSString *) c_path2
+{
+  static NSString *sqlFolderFormat
+    = (@"UPDATE %@"
+       @" SET c_path = '/'||c_path1||'/'||c_path2||'/'||c_path3||'/'||c_path4"
+       @" WHERE c_path2 = '%@'");
+
+  return [NSString stringWithFormat: sqlFolderFormat, tableName, c_path2];
+}
+
 @end
 
 //
@@ -257,7 +267,7 @@
 {
   static NSString *sqlFolderFormat
     = (@"CREATE TABLE %@ (\n"
-       @"  c_name VARCHAR (255) NOT NULL PRIMARY KEY,\n"
+       @"  c_name VARCHAR (255) PRIMARY KEY,\n"
        @"  c_content MEDIUMTEXT NOT NULL,\n"
        @"  c_creationdate INT NOT NULL,\n"
        @"  c_lastmodified INT NOT NULL,\n"
@@ -284,7 +294,7 @@
 {
   static NSString *sqlFolderFormat
     = (@"CREATE TABLE %@ (" 
-       @" c_id VARCHAR(255) NOT NULL PRIMARY KEY,"
+       @" c_id VARCHAR(255) PRIMARY KEY,"
        @" c_value VARCHAR(255) NOT NULL,"
        @" c_creationdate INT NOT NULL,"
        @" c_lastseen INT NOT NULL)");
@@ -308,6 +318,17 @@
   return types;
 }
 
+- (NSString *) updateCPathInFolderInfo: (NSString *) tableName
+                            withCPath2: (NSString *) c_path2
+{
+  static NSString *sqlFolderFormat
+    = (@"UPDATE %@"
+       @" SET c_path = CONCAT('/', c_path1, '/', c_path2, '/', c_path3, '/', c_path4)"
+       @" WHERE c_path2 = '%@'");
+
+  return [NSString stringWithFormat: sqlFolderFormat, tableName, c_path2];
+}
+
 @end
 
 //
@@ -351,7 +372,7 @@
 {
   static NSString *sqlFolderFormat
     = (@"CREATE TABLE %@ (\n"
-       @"  c_name VARCHAR2 (255) NOT NULL PRIMARY KEY,\n"
+       @"  c_name VARCHAR2 (255) PRIMARY KEY,\n"
        @"  c_content CLOB NOT NULL,\n"
        @"  c_creationdate INTEGER NOT NULL,\n"
        @"  c_lastmodified INTEGER NOT NULL,\n"
@@ -377,7 +398,7 @@
 {
   static NSString *sqlFolderFormat
     = (@"CREATE TABLE %@ (" 
-       @" c_id VARCHAR2(255) NOT NULL PRIMARY KEY,"
+       @" c_id VARCHAR2(255) PRIMARY KEY,"
        @" c_value VARCHAR2(255) NOT NULL,"
        @" c_creationdate INTEGER NOT NULL,"
        @" c_lastseen INTEGER NOT NULL)");
@@ -401,4 +422,15 @@
   return types;
 }
 
+- (NSString *) updateCPathInFolderInfo: (NSString *) tableName
+                            withCPath2: (NSString *) c_path2
+{
+  static NSString *sqlFolderFormat
+    = (@"UPDATE %@"
+       @" SET c_path = '/'||c_path1||'/'||c_path2||'/'||c_path3||'/'||c_path4"
+       @" WHERE c_path2 = '%@'");
+
+  return [NSString stringWithFormat: sqlFolderFormat, tableName, c_path2];
+}
+
 @end
