@@ -1,8 +1,6 @@
 /* SOGoMailForward.h - this file is part of SOGo
  *
- * Copyright (C) 2007 Inverse inc.
- *
- * Author: Wolfgang Sourdeau <wsourdeau@inverse.ca>
+ * Copyright (C) 2007-2013 Inverse inc.
  *
  * This file is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -39,6 +37,9 @@
 
 @end
 
+@interface SOGoMailArabicForward : SOGoMailForward
+@end
+
 @interface SOGoMailBrazilianPortugueseForward : SOGoMailForward
 @end
 
@@ -93,6 +94,9 @@
 @interface SOGoMailRussianForward : SOGoMailForward
 @end
 
+@interface SOGoMailSlovakForward : SOGoMailForward
+@end
+
 @interface SOGoMailUkrainianForward : SOGoMailForward
 @end
 
