@@ -1,8 +1,6 @@
 /* SOGoMailForward.m - this file is part of SOGo
  *
- * Copyright (C) 2007 Inverse inc.
- *
- * Author: Wolfgang Sourdeau <wsourdeau@inverse.ca>
+ * Copyright (C) 2007-2013 Inverse inc.
  *
  * This file is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -89,13 +87,13 @@
 
 - (NSString *) from
 {
-  NSString *rc;
+  id rc;
 
+  rc = [[sourceMail mailHeaders] objectForKey: @"from"];
+  if ([rc isKindOfClass: [NSArray class]])
+    rc = [rc componentsJoinedByString: @", "];
   if (htmlComposition)
-    rc = [[[sourceMail mailHeaders] objectForKey: @"from"]
-           stringByEscapingHTMLString];
-  else
-    rc = [[sourceMail mailHeaders] objectForKey: @"from"];
+    rc = [rc stringByEscapingHTMLString];
 
   return rc;
 }
@@ -119,13 +117,16 @@
 
 - (NSString *) replyTo
 {
-  NSString *rc;
+  id rc;
 
+  rc = [self _headerField: @"reply-to"];
+  if ([rc isKindOfClass: [NSArray class]])
+    rc = [rc componentsJoinedByString: @", "];
   if (htmlComposition)
     rc = [NSString stringWithFormat: @"%@<br/>", 
-          [[self _headerField: @"reply-to"] stringByEscapingHTMLString]];
+          [rc stringByEscapingHTMLString]];
   else
-    rc = ([NSString stringWithFormat: @"%@\n", [self _headerField: @"reply-to"]]);
+    rc = ([NSString stringWithFormat: @"%@\n", rc]);
 
   return rc;
 }
@@ -149,12 +150,13 @@
 
 - (NSString *) to
 {
-  NSString *rc;
+  id rc;
 
+  rc = [self _headerField: @"to"];
+  if ([rc isKindOfClass: [NSArray class]])
+    rc = [rc componentsJoinedByString: @", "];
   if (htmlComposition)
-    rc = [[[sourceMail mailHeaders] objectForKey: @"to"] stringByEscapingHTMLString];
-  else
-    rc = [[sourceMail mailHeaders] objectForKey: @"to"];
+    rc = [rc stringByEscapingHTMLString];
 
   return rc;
 }
@@ -166,13 +168,17 @@
 
 - (NSString *) cc
 {
-  NSString *rc;
+  id rc;
+
+  rc = [self _headerField: @"cc"];
+  if ([rc isKindOfClass: [NSArray class]])
+    rc = [rc componentsJoinedByString: @", "];
 
   if (htmlComposition)
     rc = [NSString stringWithFormat: @"%@<br/>", 
-          [[self _headerField: @"cc"] stringByEscapingHTMLString]];
+          [rc stringByEscapingHTMLString]];
   else
-    rc = ([NSString stringWithFormat: @"%@\n", [self _headerField: @"cc"]]);
+    rc = [NSString stringWithFormat: @"%@\n", rc];
 
   return rc;
 }
@@ -218,11 +224,15 @@
 
 - (NSString *) signature
 {
-  NSString *signature, *mailSignature;
-
+  NSString *signature, *mailSignature, *nl;
+  
   signature = [[sourceMail mailAccountFolder] signature];
+
   if ([signature length])
-    mailSignature = [NSString stringWithFormat: @"-- \n%@", signature];
+    {
+      nl = (htmlComposition ? @"<br/>" : @"\n");
+      mailSignature = [NSString stringWithFormat: @"-- %@%@", nl, signature];
+    }
   else
     mailSignature = @"";
 
@@ -231,6 +241,9 @@
 
 @end
 
+@implementation SOGoMailArabicForward
+@end
+
 @implementation SOGoMailBrazilianPortugueseForward
 @end
 
@@ -285,6 +298,9 @@
 @implementation SOGoMailRussianForward
 @end
 
+@implementation SOGoMailSlovakForward
+@end
+
 @implementation SOGoMailUkrainianForward
 @end
 
