@@ -1,8 +1,6 @@
 /* SOGoMailReply.m - this file is part of SOGo
  *
- * Copyright (C) 2007-2009 Inverse inc.
- *
- * Author: Wolfgang Sourdeau <wsourdeau@inverse.ca>
+ * Copyright (C) 2007-2013 Inverse inc.
  *
  * This file is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -101,6 +99,9 @@
 
 @end
 
+@implementation SOGoMailArabicReply
+@end
+
 @implementation SOGoMailBrazilianPortugueseReply
 @end
 
@@ -155,6 +156,9 @@
 @implementation SOGoMailRussianReply
 @end
 
+@implementation SOGoMailSlovakReply
+@end
+
 @implementation SOGoMailUkrainianReply
 @end
 
