@@ -27,6 +27,7 @@
 #import <Foundation/NSClassDescription.h>
 #import <Foundation/NSFileManager.h>
 #import <Foundation/NSPathUtilities.h>
+#import <Foundation/NSSet.h>
 #import <Foundation/NSString.h>
 #import <Foundation/NSURL.h>
 #import <Foundation/NSValue.h>
@@ -177,6 +178,9 @@
 {
   if ((self = [self init]))
     {
+      if ([_name length] == 0)
+        [NSException raise: NSInvalidArgumentException
+                     format: @"'_name' must not be an empty string"];
       context = [[WOApplication application] context];
       nameInContainer = [_name copy];
       container = _container;
@@ -1232,7 +1236,7 @@
 - (SOGoWebDAVValue *) davCurrentUserPrincipal
 {
   NSDictionary *userHREF;
-  NSString *login;
+  NSString *login, *s;
   SOGoUser *activeUser;
   SOGoWebDAVValue *davCurrentUserPrincipal;
 
@@ -1242,7 +1246,8 @@
     davCurrentUserPrincipal = nil;
   else
     {
-      userHREF = davElementWithContent (@"href", XMLNS_WEBDAV, [self davURLAsString]);
+      s = [NSString stringWithFormat: @"/SOGo/dav/%@", login];
+      userHREF = davElementWithContent (@"href", XMLNS_WEBDAV, s);
       davCurrentUserPrincipal
         = [davElementWithContent (@"current-user-principal",
                                   XMLNS_WEBDAV,
@@ -1582,6 +1587,35 @@
   return exception;
 }
 
+- (NSString *) davBooleanForResult: (BOOL) result
+{
+  return (result ? @"true" : @"false");
+}
+
+- (BOOL) isValidDAVBoolean: (NSString *) davBoolean
+{
+  static NSSet *validBooleans = nil;
+
+  if (!validBooleans)
+    {
+      validBooleans = [NSSet setWithObjects: @"true", @"false", @"1", @"0",
+                             nil];
+      [validBooleans retain];
+    }
+
+  return [validBooleans containsObject: davBoolean];
+}
+
+- (BOOL) resultForDAVBoolean: (NSString *) davBoolean
+{
+  BOOL result;
+
+  result = ([davBoolean isEqualToString: @"true"]
+            || [davBoolean isEqualToString: @"1"]);
+
+  return result;
+}
+
 - (NSString *) labelForKey: (NSString *) key
 {
   return [self labelForKey: key inContext: context];
