@@ -43,6 +43,7 @@
 - (BOOL) debugLeaks;
 - (int) vmemLimit;
 - (BOOL) trustProxyAuthentication;
+- (NSString *) encryptionKey;
 - (BOOL) useRelativeURLs;
 
 - (BOOL) isWebAccessEnabled;
@@ -76,6 +77,13 @@
 - (NSString *) CASServiceURL;
 - (BOOL) CASLogoutEnabled;
 
+- (NSString *) SAML2PrivateKeyLocation;
+- (NSString *) SAML2CertificateLocation;
+- (NSString *) SAML2IdpMetadataLocation;
+- (NSString *) SAML2IdpPublicKeyLocation;
+- (NSString *) SAML2IdpCertificateLocation;
+- (BOOL) SAML2LogoutEnabled;
+
 - (BOOL) enablePublicAccess;
 
 @end
