@@ -277,7 +277,7 @@
 
 - (void) _fetchCN
 {
-  cn = [self _fetchFieldForUser: @"cn"];
+  cn = [[self _fetchFieldForUser: @"cn"] stringByTrimmingSpaces];
   [cn retain];
 }
 
@@ -576,13 +576,13 @@
 
 - (void) _appendSystemMailAccount
 {
-  NSString *fullName, *replyTo, *imapLogin, *imapServer, *signature,
-    *encryption, *scheme, *action, *query, *customEmail;
+  NSString *fullName, *replyTo, *imapLogin, *imapServer, *cImapServer, *signature,
+    *encryption, *scheme, *action, *query, *customEmail, *sieveServer;
   NSMutableDictionary *mailAccount, *identity, *mailboxes, *receipts;
   NSNumber *port;
   NSMutableArray *identities;
   NSArray *mails;
-  NSURL *url;
+  NSURL *url, *cUrl;
   unsigned int count, max;
   NSInteger defaultPort;
 
@@ -592,8 +592,8 @@
 
   // 1. login
   imapLogin = [[SOGoUserManager sharedUserManager]
-                     getImapLoginForUID: [self loginInDomain]
-                               inDomain: [self domain]];
+                     getExternalLoginForUID: [self loginInDomain]
+                                   inDomain: [self domain]];
   [mailAccount setObject: imapLogin forKey: @"userName"];
 
   // 2. server
@@ -606,16 +606,22 @@
   // imaps://localhost:143/?tls=YES
   // imaps://localhost/?tls=YES
 
-  imapServer = [self _fetchFieldForUser: @"c_imaphostname"];
-  if (!imapServer)
-    imapServer = [[self domainDefaults] imapServer];
+  cImapServer = [self _fetchFieldForUser: @"c_imaphostname"];
+  imapServer = [[self domainDefaults] imapServer];
+  cUrl = [NSURL URLWithString: (cImapServer ? cImapServer : @"")];
   url = [NSURL URLWithString: imapServer];
-  if ([url host])
-    imapServer = [url host];
+  if([cUrl host])
+    imapServer = [cUrl host];
+  else
+    if(cImapServer)
+      imapServer = cImapServer;
+    else
+      if([url host])
+        imapServer = [url host];
   [mailAccount setObject: imapServer forKey: @"serverName"];
 
   // 3. port & encryption
-  scheme = [url scheme];
+  scheme = [cUrl scheme] ? [cUrl scheme] : [url scheme];
   if (scheme
       && [scheme caseInsensitiveCompare: @"imaps"] == NSOrderedSame)
     {
@@ -624,19 +630,28 @@
     }
   else
     {
-      query = [url query];
+      query = [cUrl query] ? [cUrl query] : [url query];
       if (query && [query caseInsensitiveCompare: @"tls=YES"] == NSOrderedSame)
         encryption = @"tls";
       else
         encryption = @"none";
       defaultPort = 143;
     }
-  port = [url port];
+  port = [cUrl port] ? [cUrl port] : [url port];
   if ([port intValue] == 0) /* port is nil or intValue == 0 */
     port = [NSNumber numberWithInt: defaultPort];
   [mailAccount setObject: port forKey: @"port"];
   [mailAccount setObject: encryption forKey: @"encryption"];
 
+  // Sieve server
+  sieveServer = [self _fetchFieldForUser: @"c_sievehostname"];
+
+  if (sieveServer)
+    {
+      [mailAccount setObject: sieveServer  forKey: @"sieveServerName"];
+    }
+  
+
   /* identities */
   identities = [NSMutableArray new];
   mails = [self allEmails];
@@ -688,7 +703,8 @@
       if (![fullName length])
         fullName = login;
       [identity setObject: fullName forKey: @"fullName"];
-      [identity setObject: [mails objectAtIndex: count] forKey: @"email"];
+      [identity setObject: [[mails objectAtIndex: count] stringByTrimmingSpaces]
+                   forKey: @"email"];
 
       if ([replyTo length] > 0)
         [identity setObject: replyTo forKey: @"replyTo"];
