@@ -1,6 +1,6 @@
 /* SOGoUserDefaults.m - this file is part of SOGo
  *
- * Copyright (C) 2009-2010 Inverse inc.
+ * Copyright (C) 2009-2012 Inverse inc.
  *
  * Author: Wolfgang Sourdeau <wsourdeau@inverse.ca>
  *
@@ -22,6 +22,7 @@
 
 #import <Foundation/NSArray.h>
 #import <Foundation/NSDictionary.h>
+#import <Foundation/NSSet.h>
 #import <Foundation/NSString.h>
 #import <Foundation/NSTimeZone.h>
 
@@ -91,6 +92,22 @@ NSString *SOGoWeekStartFirstFullWeek = @"FirstFullWeek";
   return ud;
 }
 
+- (id) init
+{
+  if ((self = [super init]))
+    {
+      userLanguage = nil;
+    }
+
+  return self;
+}
+
+- (void) dealloc
+{
+  [userLanguage release];
+  [super dealloc];
+}
+
 - (BOOL) _migrateLastModule
 {
   BOOL rc;
@@ -243,16 +260,6 @@ NSString *SOGoWeekStartFirstFullWeek = @"FirstFullWeek";
   return [self stringForKey: @"SOGoDefaultCalendar"];
 }
 
-- (void) setAppointmentSendEMailReceipts: (BOOL) newValue
-{
-  [self setBool: newValue forKey: @"SOGoAppointmentSendEMailReceipts"];
-}
-
-- (BOOL) appointmentSendEMailReceipts
-{
-  return [self boolForKey: @"SOGoAppointmentSendEMailReceipts"];
-}
-
 - (void) setDayStartTime: (NSString *) newValue
 {
   [self setObject: newValue forKey: @"SOGoDayStartTime"];
@@ -365,21 +372,25 @@ NSString *SOGoWeekStartFirstFullWeek = @"FirstFullWeek";
 
 - (NSString *) language
 {
-  NSString *language;
   NSArray *supportedLanguages;
-  
-  /* see SOGoDomainDefaults for the meaning of this */
-  language = [source objectForKey: @"SOGoLanguage"];
-  if (!(language && [language isKindOfClass: [NSString class]]))
-    language = [(SOGoDomainDefaults *) parentSource language];
-  
-  /* make sure the language is part of the supported languages */
-  supportedLanguages = [[SOGoSystemDefaults sharedSystemDefaults]
-                         supportedLanguages];
-  if (![supportedLanguages containsObject: language])
-    language = [parentSource stringForKey: @"SOGoLanguage"];
 
-  return language;
+  if (!userLanguage)
+    {
+      /* see SOGoDomainDefaults for the meaning of this */
+      userLanguage = [source objectForKey: @"SOGoLanguage"];
+      if (!(userLanguage && [userLanguage isKindOfClass: [NSString class]]))
+        userLanguage = [(SOGoDomainDefaults *) parentSource language];
+  
+      supportedLanguages = [[SOGoSystemDefaults sharedSystemDefaults]
+                             supportedLanguages];
+
+      /* make sure the language is part of the supported languages */
+      if (![supportedLanguages containsObject: userLanguage])
+        userLanguage = [parentSource stringForKey: @"SOGoLanguage"];
+      [userLanguage retain];
+    }
+
+  return userLanguage;
 }
 
 - (void) setMailShowSubscribedFoldersOnly: (BOOL) newValue
@@ -482,6 +493,16 @@ NSString *SOGoWeekStartFirstFullWeek = @"FirstFullWeek";
   return [self stringForKey: @"SOGoMailComposeMessageType"];
 }
 
+- (void) setMailDisplayRemoteInlineImages: (NSString *) newValue;
+{
+  [self setObject: newValue forKey: @"SOGoMailDisplayRemoteInlineImages"];
+}
+
+- (NSString *) mailDisplayRemoteInlineImages;
+{
+  return [self stringForKey: @"SOGoMailDisplayRemoteInlineImages"];
+}
+
 - (void) setMailMessageForwarding: (NSString *) newValue
 {
   [self setObject: newValue forKey: @"SOGoMailMessageForwarding"];
@@ -537,7 +558,8 @@ NSString *SOGoWeekStartFirstFullWeek = @"FirstFullWeek";
 {
   if ([newValue length] == 0)
     newValue = nil;
-  [self setObject: newValue forKey: @"SOGoMailCustomFullName"];
+  [self setObject: [newValue stringByTrimmingSpaces]
+           forKey: @"SOGoMailCustomFullName"];
 }
 
 - (NSString *) mailCustomFullName
@@ -549,7 +571,8 @@ NSString *SOGoWeekStartFirstFullWeek = @"FirstFullWeek";
 {
   if ([newValue length] == 0)
     newValue = nil;
-  [self setObject: newValue forKey: @"SOGoMailCustomEmail"];
+  [self setObject: [newValue stringByTrimmingSpaces]
+           forKey: @"SOGoMailCustomEmail"];
 }
 
 - (NSString *) mailCustomEmail
@@ -561,7 +584,8 @@ NSString *SOGoWeekStartFirstFullWeek = @"FirstFullWeek";
 {
   if ([newValue length] == 0)
     newValue = nil;
-  [self setObject: newValue forKey: @"SOGoMailReplyTo"];
+  [self setObject: [newValue stringByTrimmingSpaces]
+           forKey: @"SOGoMailReplyTo"];
 }
 
 - (NSString *) mailReplyTo
