@@ -25,6 +25,7 @@
 #import <Foundation/NSEnumerator.h>
 #import <Foundation/NSException.h>
 #import <Foundation/NSLock.h>
+#import <Foundation/NSNull.h>
 #import <Foundation/NSString.h>
 #import <Foundation/NSTimer.h>
 #import <Foundation/NSValue.h>
@@ -43,8 +44,15 @@
 #import "SOGoConstants.h"
 #import "SOGoSource.h"
 
+static Class NSNullK;
+
 @implementation SOGoUserManagerRegistry
 
++ (void) initialize
+{
+  NSNullK = [NSNull class];
+}
+
 + (id) sharedRegistry
 {
   static id sharedRegistry = nil;
@@ -350,8 +358,8 @@
   return fullEmail;
 }
 
-- (NSString *) getImapLoginForUID: (NSString *) uid
-                         inDomain: (NSString *) domain
+- (NSString *) getExternalLoginForUID: (NSString *) uid
+                             inDomain: (NSString *) domain
 {
   NSDictionary *contactInfos;
   NSString *login;
@@ -367,7 +375,7 @@
       else
         dd = [SOGoSystemDefaults sharedSystemDefaults];
       
-      login = [dd forceIMAPLoginWithEmail] ? [self getEmailForUID: uid] : uid;
+      login = [dd forceExternalLoginWithEmail] ? [self getEmailForUID: uid] : uid;
     }
   
   return login;
@@ -440,11 +448,28 @@
 }
 
 - (BOOL) checkLogin: (NSString *) _login
-	   password: (NSString *) _pwd
+           password: (NSString *) _pwd
              domain: (NSString **) _domain
-	       perr: (SOGoPasswordPolicyError *) _perr
-	     expire: (int *) _expire
-	      grace: (int *) _grace
+               perr: (SOGoPasswordPolicyError *) _perr
+             expire: (int *) _expire
+              grace: (int *) _grace
+{
+  return [self checkLogin: _login
+                   password: _pwd
+                   domain: _domain
+                   perr: _perr
+                   expire: _expire
+                   grace: _grace
+                   useCache: YES];
+}
+
+- (BOOL) checkLogin: (NSString *) _login
+           password: (NSString *) _pwd
+             domain: (NSString **) _domain
+               perr: (SOGoPasswordPolicyError *) _perr
+             expire: (int *) _expire
+              grace: (int *) _grace
+           useCache: (BOOL) useCache
 {
   NSString *dictPassword, *username, *jsonUser;
   NSMutableDictionary *currentUser;
@@ -460,7 +485,7 @@
   jsonUser = [[SOGoCache sharedCache] userAttributesForLogin: username];
   currentUser = [jsonUser objectFromJSONString];
   dictPassword = [currentUser objectForKey: @"password"];
-  if (currentUser && dictPassword)
+  if (useCache && currentUser && dictPassword)
     {
       checkOK = ([dictPassword isEqualToString: [_pwd asSHA1String]]);
       //NSLog(@"Password cache hit for user %@", _login);
@@ -474,9 +499,9 @@
     {
       checkOK = YES;
       if (!currentUser)
-	{
-	  currentUser = [NSMutableDictionary dictionary];
-	}
+        {
+          currentUser = [NSMutableDictionary dictionary];
+        }
 
       // It's important to cache the password here as we might have cached the
       // user's entry in -contactInfosForUserWithUIDorEmail: and if we don't
@@ -587,7 +612,7 @@
 		   withUIDorEmail: (NSString *) uid
                          inDomain: (NSString *) domain
 {
-  NSString *sourceID, *cn, *c_domain, *c_uid, *c_imaphostname, *c_imaplogin;
+  NSString *sourceID, *cn, *c_domain, *c_uid, *c_imaphostname, *c_imaplogin, *c_sievehostname;
   NSObject <SOGoSource> *currentSource;
   NSEnumerator *sogoSources;
   NSDictionary *userEntry;
@@ -602,6 +627,7 @@
   c_domain = nil;
   c_imaphostname = nil;
   c_imaplogin = nil;
+  c_sievehostname = nil;
 
   [currentUser setObject: [NSNumber numberWithBool: YES]
 	       forKey: @"CalendarAccess"];
@@ -632,6 +658,8 @@
 	    c_imaphostname = [userEntry objectForKey: @"c_imaphostname"];
           if (!c_imaplogin)
             c_imaplogin = [userEntry objectForKey: @"c_imaplogin"];
+          if (!c_sievehostname)
+            c_sievehostname = [userEntry objectForKey: @"c_sievehostname"];
 	  access = [[userEntry objectForKey: @"CalendarAccess"] boolValue];
 	  if (!access)
 	    [currentUser setObject: [NSNumber numberWithBool: NO]
@@ -667,6 +695,8 @@
     [currentUser setObject: c_imaphostname forKey: @"c_imaphostname"];
   if (c_imaplogin)
     [currentUser setObject: c_imaplogin forKey: @"c_imaplogin"];
+  if (c_sievehostname)
+    [currentUser setObject: c_sievehostname forKey: @"c_sievehostname"];
 
   [currentUser setObject: emails forKey: @"emails"];
   [currentUser setObject: cn forKey: @"cn"];
@@ -691,18 +721,20 @@
   [[SOGoCache sharedCache]
         setUserAttributes: [newUser jsonRepresentation]
                  forLogin: login];
-  
-  key = [newUser objectForKey: @"c_uid"];
-  if (key && ![key isEqualToString: login])
-    [[SOGoCache sharedCache]
-        setUserAttributes: [newUser jsonRepresentation]
-                 forLogin: key];
+  if (![newUser isKindOfClass: NSNullK])
+    {
+      key = [newUser objectForKey: @"c_uid"];
+      if (key && ![key isEqualToString: login])
+        [[SOGoCache sharedCache]
+            setUserAttributes: [newUser jsonRepresentation]
+                     forLogin: key];
 
-  emails = [[newUser objectForKey: @"emails"] objectEnumerator];
-  while ((key = [emails nextObject]))
-    [[SOGoCache sharedCache]
-        setUserAttributes: [newUser jsonRepresentation]
-                 forLogin: key];
+      emails = [[newUser objectForKey: @"emails"] objectEnumerator];
+      while ((key = [emails nextObject]))
+        [[SOGoCache sharedCache]
+            setUserAttributes: [newUser jsonRepresentation]
+                     forLogin: key];
+    }
 }
 
 - (NSMutableDictionary *) _contactInfosForAnonymous
@@ -787,7 +819,9 @@
         cacheUid = aUID;
       jsonUser = [[SOGoCache sharedCache] userAttributesForLogin: cacheUid];
       currentUser = [jsonUser objectFromJSONString];
-      if (!([currentUser objectForKey: @"emails"]
+      if ([currentUser isKindOfClass: NSNullK])
+        currentUser = nil;
+      else if (!([currentUser objectForKey: @"emails"]
 	    && [currentUser objectForKey: @"cn"]))
 	{
 	  // We make sure that we either have no occurence of a cache entry or
@@ -808,11 +842,15 @@
                                 inDomain: domain];
 	  if (newUser)
 	    {
-	      if ([[currentUser objectForKey: @"c_uid"] length] > 0)
-		[self _retainUser: currentUser
+	      if ([[currentUser objectForKey: @"c_uid"] length] == 0)
+                {
+                  [self _retainUser: (NSDictionary *) [NSNull null]
+                          withLogin: cacheUid];
+                  currentUser = nil;
+                }
+              else
+                [self _retainUser: currentUser
                         withLogin: cacheUid];
-	      else
-		currentUser = nil;
 	    }
 	}
     }
