@@ -46,6 +46,7 @@
 #import <SOGo/SOGoUserFolder.h>
 #import <SOGo/WOResourceManager+SOGo.h>
 #import <Mailer/SOGoMailAccount.h>
+#import <Mailer/SOGoMailAccounts.h>
 
 #import "UIxPreferences.h"
 
@@ -63,25 +64,19 @@
 {
   //NSDictionary *locale;
   SOGoDomainDefaults *dd;
-  NSString *language;
   
   if ((self = [super init]))
     {
       item = nil;
 #warning user should be the owner rather than the activeUser
       ASSIGN (user, [context activeUser]);
-      ASSIGN (userDefaults, [user userDefaults]);
       ASSIGN (today, [NSCalendarDate date]);
-      //locale = [context valueForKey: @"locale"];
-      language = [userDefaults language];
 
       calendarCategories = nil;
       calendarCategoriesColors = nil;
       defaultCategoryColor = nil;
       category = nil;
 
-      ASSIGN (locale,
-	      [[self resourceManager] localeForLanguageNamed: language]);
       ASSIGN (daysOfWeek, [locale objectForKey: NSWeekDayNameArray]);
 
       dd = [user domainDefaults];
@@ -119,7 +114,6 @@
   [today release];
   [item release];
   [user release];
-  [userDefaults release];
   [sieveFilters release];
   [vacationOptions release];
   [calendarCategories release];
@@ -129,7 +123,6 @@
   [contactsCategories release];
   [forwardOptions release];
   [daysOfWeek release];
-  [locale release];
   [super dealloc];
 }
 
@@ -766,6 +759,28 @@
   [userDefaults setMailComposeMessageType: newType];
 }
 
+/* Display remote inline images */
+- (NSArray *) displayRemoteInlineImages
+{
+  return [NSArray arrayWithObjects: @"never", @"always", nil];
+}
+
+- (NSString *) itemDisplayRemoteInlineImagesText
+{
+  return [self labelForKey: [NSString stringWithFormat:
+                                        @"displayremoteinlineimages_%@", item]];
+}
+
+- (NSString *) userDisplayRemoteInlineImages
+{
+  return [userDefaults mailDisplayRemoteInlineImages];
+}
+
+- (void) setUserDisplayRemoteInlineImages: (NSString *) newType
+{
+  [userDefaults setMailDisplayRemoteInlineImages: newType];
+}
+
 /* mail autoreply (vacation) */
 
 - (BOOL) isSieveScriptsEnabled
